#!/usr/bin/perl

use strict;

#############################################
#
# FITSplot.pl v2.03, May 2017
#
# --> commented out getting
#     beam info from history
#     file (rarely needed) as 
#     this no longer works in newer perls
#
# FITSplot.pl v2.02, October 2002
#
# Subroutines for reading and plotting
# FITS images with PGPLOT.  The perl
# data language PDL takes a primary 
# role in making for fast computations.
#
# See the "README" file to get started.  
#   
# See files example*.pl to see ways
# these subroutines might be used.
#
# - Dan Homan, October 2002
#   dhoman@nrao.edu
#
#############################################

use PGPLOT;
use PDL;
use PDL::Math;
use PDL::Image2D;
use PDL::Graphics::LUT;
use PDL::Graphics::PGPLOT;
use Carp;
use strict;

###########################################
#
#  Setting up default values for plotting
#
###########################################

our %plot;                     # global hash variable with defaults

#------------------------- defining map units ---------------------------#
$plot{'units'} = 'auto';         # default map units.
                                 #  'auto' means determine automatically
                                 #  with the first Load_FITS command.
                                 #  See 'factor' (next variable) for 
                                 #  possible unit choices.

our %factor;                     # conversion factor from deg to map units 
    $factor{'marcsec'} = 3.6e6;  
    $factor{'mas'}     = 3.6e6;  # mas as an alias for marcsec 
    $factor{'arcsec'}  = 3.6e3;
    $factor{'arcmin'}  = 60;           

#---------------- how much of an image should be plotted ----------------#
$plot{'image_corners'} = [0,0,0,0]; # x1,x2,y1,y2 for the image 
                                    #   in map units. All zeros means
                                    #   use full image
$plot{'plot_corners'} = [0,0,0,0];  # x1,x2,y1,y2 for the plotting
                                    #   frame.  All zeros means use
                                    #   the image corners.

#------------- how maps should be arranged on 'the page' ----------------#
$plot{'offset'} = [0,0];   # offset for placing the plot on
                           #  'the page' in map units
$plot{'rotate'} = 0;       # amount to rotate image on the page (deg)

#------------- how polarization vector should be plotted ----------------#
$plot{'tick_sep'} = [3,3]; # interval between pol. ticks (pixels)
$plot{'tick_length'} = 1;  # scale length of pol. ticks in map units
$plot{'tick_rotate'} = 0;  # additional rotation of pol. ticks (deg)

#-------------------- how the colormap should appear --------------------#
$plot{'color_type'} = 'heat';  # color scheme to use
$plot{'color_reverse'} = 0;    # set to one to reverse color ramp
$plot{'color_ramp'} = 'ramp';  # type of color ramp
$plot{'color_contrast'} = 1.0; # contrast level
$plot{'color_bright'} = 0.5;   # brightness (between 0 and 1)

$plot{'pixel_range'} = [0,0];  # min,max range for plotting color/grayscale
                               #   All zeros means use full range of image.

$plot{'wedge'} = 1;            # set to zero for no color wedge
$plot{'wedge_side'} = 'T';     # "B","L","T","R" for Bottom, Left, Top, 
                               #   or Right side where the wedge is plotted
$plot{'wedge_disp'} = 0;       # displacement of wedge in character heights
                               #   from the side indicated above.  Negative
                               #   values will place the plot inside the
                               #   viewport
$plot{'wedge_width'} = 3;      # width of the color wedge in character heights
$plot{'wedge_label'} = '';     # string with a label for the wedge

#--------------------- how contour plots should look --------------------#
$plot{'contour_range'} = 5e4;   # dynamic range of contours produced by
                                #   the 'loglevs' function
$plot{'contour_negative'} = 2;  # whether or not to make an equal number 
                                #   negative and positive contours. 
                                #   0 -> no negative contours
                                #   1 -> just 1 negative contour
                                #   2 -> a negative contour for every
                                #        positive one   

#------------------------  Misc defaults --------------------------------#
$plot{'title'} = '';           # title for the plot
$plot{'font'} = 2;             # default font.  1 = normal, 2 = roman
                               #                3 = italics, 4 = script
 
#######################################
#
# Load a FITS file and 'slice' it to
# the image coordinates of interest,
# while maintaining correct header 
# information.
#
#######################################

sub Load_FITS {

 my $file = $_[0];    # file to load 
 my $plane1 = $_[1];  # which plane on the third axis of FITS image?
 my $plane2 = $_[2];  # which plane on the fourth axis of FITS image?
 if(!defined($plane1)) { $plane1 = 0; }   # default to first plane
 if(!defined($plane2)) { $plane2 = 0; }   # default to first plane

 my($minx, $maxx, $miny, $maxy, $h, $g, $degpp);
 my($f,$centx, $centy, $axisx, $axisy, $naxis);

 $f = rfits($file);           # load the file as a piddle

 $h = $f->gethdr;             # get fits header info
 $naxis = $$h{NAXIS};         # number of axis in file
 $degpp = $$h{CDELT2};        # degrees per pixel 
 $centx = $$h{CRPIX1};        # center pixel in x
 $centy = $$h{CRPIX2};        # center pixel in y
 $axisx = $$h{NAXIS1};        # pixels along x axis
 $axisy = $$h{NAXIS2};        # pixels along y axis

 # Determine plot size units based on cellspacing
 if($plot{'unit'} == 'auto') {
   if ($degpp < 1.0/360000.0)  {$plot{'units'}='marcsec';}  # mas
   elsif ($degpp < 1.0/6000.0) {$plot{'units'}='arcsec';}   # asec
   else  {$plot{'units'}='arcmin';}                         # amin
 }

 my $convert = $factor{$plot{'units'}};   # degrees to plot units
 $convert *= $degpp;          # conversion is now pixels to plot units

 # define image corners for area of interest

 $minx = $plot{'image_corners'}->[0];
 $maxx = $plot{'image_corners'}->[1];
 $miny = $plot{'image_corners'}->[2];
 $maxy = $plot{'image_corners'}->[3];

 if($minx != 0 || $maxx != 0 || $miny != 0 || $maxy != 0) {
   $minx = long($centx - $minx/$convert); 
   if($minx < 0) { $minx = 0; }
   $maxx = long($centx - $maxx/$convert);
   if($maxx >= $axisx) { $maxx = $axisx-1; }
   $miny = long($centy + $miny/$convert);
   if($miny < 0) { $miny = 0; }
   $maxy = long($centy + $maxy/$convert);
   if($maxy >= $axisy) { $maxy = $axisy-1; }
 } else {
   # use full image
   $minx = 0; $maxx = $axisx-1;
   $miny = 0; $maxy = $axisy-1;
   $plot{'image_corners'}->[0] = ($centx-$minx)*$convert;
   $plot{'image_corners'}->[1] = ($centx-$maxx)*$convert;
   $plot{'image_corners'}->[2] = ($miny-$centy)*$convert;
   $plot{'image_corners'}->[3] = ($maxy-$centy)*$convert;
 } 

 # slice the image down to size

 if($naxis == 4) {
  $g = $f->slice("$minx:$maxx,$miny:$maxy,$plane1,$plane2"); # slice the piddle
 }

 if($naxis == 3) {
  $g = $f->slice("$minx:$maxx,$miny:$maxy,$plane1");  # slice the piddle
 }

 if($naxis == 2) {
  $g = $f->slice("$minx:$maxx,$miny:$maxy");  # slice the piddle
 }

 # fix up new header information
 $centx -= $minx;
 $centy -= $miny;
 $maxx++; $maxy++;
 $$h{NAXIS1} = $maxx-$minx;     # set axis to appropriate size
 $$h{NAXIS2} = $maxy-$miny;
 $$h{CRPIX1} = "$centx";        # fix the pixel value of center    
 $$h{CRPIX2} = "$centy";
 $g->sethdr($h);

 undef $file; undef $convert;
 undef $minx; undef $maxx; undef $miny; undef $maxy;
 undef $h; undef $degpp; undef $centx; undef $f; 
 undef $centy; undef $axisx; undef $axisy; undef $naxis; 
  
 return($g);

}

#######################################
#
# Subroutine to plot a beam figure
#
#######################################

sub plotbeam {

 # Arguments

 my $Array = $_[0];
 my $cent  = $_[1];

 my $rotate = $plot{'rotate'};

 my($bmaj, $bmin, $bpa, $h);

 $h = $Array->gethdr;
 $bmaj = $$h{BMAJ};           # major axis of beam
 $bmin = $$h{BMIN};           # minor axis of beam
 $bpa  = $$h{BPA};            # position angle of beam

 # might need to parse the history file for the beam
# if(!defined($bmaj)) {
#  my ($c, $d, $e, $i);  $i = 0;
#  while($$h{HISTORY}->[$i]) {
#   if($$h{HISTORY}->[$i] =~ /BMAJ/) {
#    ($a, $b, $c) = split(/CLEAN/,$$h{HISTORY}->[$i]);
#    ($a, $c) = split(/HISTORY/, $b);
#    ($b, $c, $bmaj, $d, $bmin, $e, $bpa) = split(/[B=]/, $a);
#    # print "$a\n";
#   } 
#   $i++;
#  }
#  undef $a; undef $b; undef $c; undef $d; undef $e; undef $i; 
# }

 if(!defined($bmaj)) {
     print "No Beam information found in FITS file. Beam not plotted.\n";
     return;
 }


 # convert to plotting units
 $bmaj *= $factor{$plot{'units'}};
 $bmin *= $factor{$plot{'units'}};

 # use the bottom left corner of plot if center location
 # for the beam is not defined

 if(!defined($cent)) {
   if($plot{'plot_corners'}->[0] == 0 &&
      $plot{'plot_corners'}->[1] == 0 &&
      $plot{'plot_corners'}->[2] == 0 &&
      $plot{'plot_corners'}->[3] == 0) {
       $cent->[0] = $plot{'image_corners'}->[0]-$bmaj;
       $cent->[1] = $plot{'image_corners'}->[2]+$bmaj;
   } else {   
       $cent->[0] = $plot{'plot_corners'}->[0]-$bmaj;
       $cent->[1] = $plot{'plot_corners'}->[2]+$bmaj;
   }
 }

 my ($delx, $dely, $x1, $x2, $y1, $y2, $ang);

 $ang = 3.14159*($bpa+$rotate)/180.0;  
 $delx = $bmaj*sin($ang)/2.0;
 $dely = $bmaj*cos($ang)/2.0;
 $x1 = $delx+$cent->[0];
 $x2 = -$delx+$cent->[0];
 $y1 = $dely+$cent->[1];
 $y2 = -$dely+$cent->[1];
 pgmove($x1, $y1);
 pgdraw($x2, $y2);

 $ang = 3.14159*($bpa+$rotate+90)/180.0;  
 $delx = $bmin*sin($ang)/2.0;
 $dely = $bmin*cos($ang)/2.0;
 $x1 = $delx+$cent->[0];
 $x2 = -$delx+$cent->[0];
 $y1 = $dely+$cent->[1];
 $y2 = -$dely+$cent->[1];
 pgmove($x1, $y1);
 pgdraw($x2, $y2);

 undef $cent; undef $bmaj; undef $bmin;
 undef $bpa; undef $rotate; undef $Array;

}


#######################################################
#
# Subroutine to generate P and Chi from Stokes Q and U
#
#######################################################

sub MakeP {

 # Arguments

 my $Q = $_[0];       # Stokes Q image piddle
 my $U = $_[1];       # Stokes U image piddle
 my ($p, $chi);

 #
 # Generate P and Chi 
 #
 
 $p = sqrt($U*$U+$Q*$Q);   
 $chi = (180/3.14159)*atan2($U,$Q)/2.0;

 $p->sethdr($Q->gethdr);
 $chi->sethdr($Q->gethdr);

 $_[2] = $p;
 $_[3] = $chi;

 undef $Q; undef $U; 

 return ($p, $chi);  # return polarization images 
 
}

#######################################################
#
# Subroutine to generate fractional polarization, blank
#  value are set to 0 
#
#######################################################

sub MakeF {

 # Arguments

 my $I = $_[0];       # Stokes I image array
 my $P = $_[1];       # Stokes P image array
 my $icut  = $_[2];   # cut level for I
 my $pcut  = $_[3];   # cut level for P
 
 my $f;

 #
 # Generate Fractional Polarization 
 #   -- masks are so that we can set
 #      the pixels below cutoff to zero
 #

 my $mask1 = long((atan($I/$icut)/(2.0*atan(1.0)) + 1.0)/1.5);
 my $mask2 = long((atan($P/$pcut)/(2.0*atan(1.0)) + 1.0)/1.5);
 
 # Return the fractional polarization map

 $f = ($P/$I)*($mask1&$mask2);# - 100.0*(1^($mask1&$mask2));
 $f->sethdr($I->gethdr);
  
 undef $I; undef $P; undef $icut; undef $pcut;
 undef $mask1; undef $mask2;  

 return($f);           # return fractional polarization
}

#######################################################
#
# Subroutine to generate spectral index map, blank
#  value are set to -100 
#
#######################################################

sub MakeAlpha {

 # Arguments

 my $I1 = $_[0];       # Stokes I1 image array
 my $I2 = $_[1];       # Stokes I2 image array
 my $icut1  = $_[2];   # cut level for I1
 my $icut2  = $_[3];   # cut level for I2

 my $freq1;            # frequency for I1
 my $freq2;            # frequency for I2

 my $h1; my $h2; my $temp;
 my $a;                # spectral index array   

 $h1 = $I1->gethdr;
 $h2 = $I2->gethdr;

 $freq1 = $$h1{CRVAL3};
 $freq2 = $$h2{CRVAL3};

 #
 # Generate Spectral Index
 #   -- masks are so that we can set
 #      the pixels below cutoff to -100
 #

 my $mask1 = long((atan($I1/$icut1)/(2.0*atan(1.0)) + 1.0)/1.5);
 my $mask2 = long((atan($I2/$icut2)/(2.0*atan(1.0)) + 1.0)/1.5);
 
 # Return the spectral index map

 $temp = ($I1/$I2)*($mask1&$mask2) + 1000.0*(1^($mask1&$mask2));
 $a = float(log $temp)/float(log($freq1/$freq2));
 $a->sethdr($h2);
  
 undef $I1; undef $I2; undef $icut1; undef $icut2;
 undef $freq1; undef $freq2; undef $h1; undef $h2;
 undef $mask1; undef $mask2; undef $temp;

 return($a);          # return spectral index

}

####################################################
#
# Subroutine to linearly shift an image in (x,y).
#  This is not just in display, but the actual
#  image pixel values.  Interpolation is used.
#  Blank values are set to zero.
#
####################################################
sub Shift {

  my $image = $_[0];     # image piddle
  my $xshift = $_[1];    # shift in X (plot units)
  my $yshift = $_[2];    # shift in Y (plot units)

  my $convert = $factor{$plot{'units'}};   # degrees to plot units

  my $px; my $py;        # tranform arrays
  my $temp;              # temporary image piddle
   
  my($h, $degpp);

  $h = $image->gethdr;         # get fits header info
  $degpp = $$h{CDELT2};        # degrees per pixel 
  $convert *= $degpp;          # conversion is map units per pixel

  $xshift = -$xshift/$convert;
  $yshift = $yshift/$convert;

  # transformation image (x1,y1) to (x2,y2)
  #  x1 = $xshift + 1.0*x2 + 0.0*y2 + 0.0*x2*y2 
  #  y1 = $yshift + 0.0*x2 + 1.0*y2 + 0.0*x2*y2 
  $px = pdl([$xshift,1.0],[0.0,0.0]);
  $py = pdl([$yshift,0.0],[1.0,0.0]);
  $temp = warp2d($image,$px,$py, {KERNEL=>"tanh", NOVAL=>0});
  
  $temp->sethdr($h);
  
  $_[0] = $temp;

  undef $image; undef $xshift; undef $yshift;
  undef $px; undef $py; undef $temp; undef $h;
  undef $degpp;
}

####################################################
#
# Subroutine to get FITS file info and consturct
#  a transformation array used by PGPLOT's routines
#
####################################################
sub Get_Trans {

 my $array = $_[0];      # image piddle
 my $convert = $_[1];    # degrees to plot units

 my $shift = $plot{'offset'};   # shift image in (X,Y) - plot units
 my $rotate = $plot{'rotate'};  # rotate image (degrees)

 $rotate *= 3.14159/180.0;   # convert to radians

 my($h, $degpp);
 my($centx, $centy, $axisx, $axisy);

 $h = $array->gethdr;        # get fits header info
 $degpp = $$h{CDELT2};        # degrees per pixel 
 $centx = $$h{CRPIX1};        # center pixel in x
 $centy = $$h{CRPIX2};        # center pixel in y
 $axisx = $$h{NAXIS1};        # pixels along x axis
 $axisy = $$h{NAXIS2};        # pixels along y axis
 $convert *= $degpp;          # conversion is now pixels to world units

 #
 # compute tranform...
 #

 my $tr;

 $tr->[0] = $shift->[0]+($centx)*$convert*cos($rotate)
                   +($centy)*$convert*sin($rotate);
 $tr->[1] = -$convert*cos($rotate);
 $tr->[2] = -$convert*sin($rotate);
 $tr->[3] = $shift->[1]+($centx)*$convert*sin($rotate)
                   -($centy)*$convert*cos($rotate);
 $tr->[4] = -$convert*sin($rotate);
 $tr->[5] = $convert*cos($rotate);

 #
 # Return stuff
 #
 
 $_[1] = $convert;             # now goes from pixels to world units 
 $_[2] = $tr;                  # PGPLOT transform array
 $_[3] = $centx;
 $_[4] = $centy;
 $_[5] = $axisx;
 $_[6] = $axisy; 

 undef $array; undef $convert; undef $h; undef $degpp;
 undef $centx; undef $centy; undef $axisx; undef $axisy;
 undef $tr; undef $shift; undef $rotate;

}

####################################################
#
# Subroutine for plotting polarization ticks
#
####################################################

sub ticks {

 # Arguments 

 my $Parray = $_[0];       # P image piddle
 my $Xarray = $_[1];       # Chi image piddle (degrees)
 my $cut_level = $_[2];    # level to cut at in P image (in mJy)
 my $Larray = $_[3];       # relative tick length piddle

 my $convert = $factor{$plot{'units'}};   # degrees to plot units

 my $tick_sep = $plot{'tick_sep'};        # tick separation (X, Y) (pixels) 
 my $length = $plot{'tick_length'};       # scale length of ticks (world units)
 my $evpa_corr = $plot{'tick_rotate'};    # evpa rotation to apply (degrees) 

 my($centx, $centy, $axisx, $axisy, $tr);

  # Get file info and tranformation information,
  # --  note that $convert is returned as a value that 
  #     converts from pixels to world units
 Get_Trans($Parray, $convert, $tr, $centx, $centy, $axisx, $axisy);

 #
 # Plot it up...
 #

 my ($ang, $delx, $dely, $x1, $x2, $y1, $y2, $tick_size);
 
 # Tick drawing adapted from Scott Aaron's C++ program to make tick marks 

 for($a=0; $a < $axisx; $a += $tick_sep->[0]) {
  for($b=0; $b < $axisy; $b += $tick_sep->[1]) { 
   if($Parray->at($a,$b,0,0) >= $cut_level) {
    if(!defined($Larray)) { $tick_size = $length; }
    else { $tick_size = $length*$Larray->at($a,$b,0,0); }   
    $ang = 3.14159*($Xarray->at($a,$b,0,0)+$evpa_corr)/180.0;  
    $delx = $tick_size*sin($ang)/2.0;
    $dely = $tick_size*cos($ang)/2.0;
    $x1 = $delx+$tr->[0]+$tr->[1]*($a+1)+$tr->[2]*($b+1);
    $x2 = -$delx+$tr->[0]+$tr->[1]*($a+1)+$tr->[2]*($b+1);
    $y1 = $dely+$tr->[3]+$tr->[4]*($a+1)+$tr->[5]*($b+1);
    $y2 = -$dely+$tr->[3]+$tr->[4]*($a+1)+$tr->[5]*($b+1);
    pgmove($x1, $y1);
    pgdraw($x2, $y2);
   }
  }
 }

 undef $ang; undef $delx; undef $dely; undef $x1; undef $x2;
 undef $y1; undef $y2; undef $tr;
 undef $Parray; undef $Xarray; undef $Larray; 
 undef $centx; undef $centy; undef $tick_sep; 
 undef $length; undef $evpa_corr; undef $cut_level;
 undef $axisx; undef $axisy; undef $convert;

}


###################################################
#
# Define simple contour levels
#
###################################################
sub loglevs {
 
 my $base = $_[0];
 my $step = $_[1];

 # setup the maximum dynamic range
 my $maxlev = int(log($plot{'contour_range'})/log($step));
 
 my (@conts, $levs);

 if($plot{'contour_negative'} == 2) {

  if(($maxlev&1)) { $maxlev += 1; }  # ensure it is even

  $conts[$maxlev-1] = -$base;
  $conts[$maxlev] = $base;
 
  for($levs = 1; $levs < $maxlev; $levs++) {   
   $conts[$maxlev-1-$levs] = $step*$conts[$maxlev-$levs];
   $conts[$maxlev+$levs] = $step*$conts[$maxlev+$levs-1];
  }

 } elsif($plot{'contour_negative'} == 1) {

  $conts[0] = -$base;
  $conts[1] = $base;
 
  for($levs = 2; $levs <= $maxlev; $levs++) {   
   $conts[$levs] = $step*$conts[$levs-1];
  }

 } else {

  $conts[0] = $base;
 
  for($levs = 1; $levs < $maxlev; $levs++) {   
   $conts[$levs] = $step*$conts[$levs-1];
  }
 }


 return @conts;

}


###################################################
#
# Subroutine for plotting a FITS image in contours
#
###################################################

sub contour {

 # Arguments 

 my $array = $_[0]; # image array
 my $conts = $_[1];        # array of contour levels
 my $nlevs = $_[2];        # number of levels in array

 my $convert = $factor{$plot{'units'}};   # degrees to plot units

 # truncate the contour array to the range of the image
 #  -- this should speed up plotting in many cases --
 my $base = min($array);  my $peak = max($array);
 my $count1 = 0; my $count2 = 0; my @contours;
 while($count1 < $nlevs) {
  if($conts->[$count1] > $peak) { last; }
  if($conts->[$count1] > $base) {
   $contours[$count2] = $conts->[$count1];
   $count2++;
  }
  $count1++;
 }
 
 my($centx, $centy, $axisx, $axisy, $tr);

  # Get file info and tranformation information,
  # --  note that $convert is returned as a value that 
  #     converts from pixels to world units
 Get_Trans($array, $convert, $tr, $centx, $centy, $axisx, $axisy);

 $array = float($array);  # convert to float to be sure

 #
 # Plot it up...
 #

 pgcont($array->get_dataref,$axisx,$axisy,1,$axisx,1,$axisy, 
   \@contours,$count2,$tr);

 undef $array; undef @contours; undef $base; undef $peak;
 undef $count1; undef $count2; 
 undef $axisx; undef $axisy; undef $centx; undef $centy;
 undef $nlevs; undef $conts; undef $tr; undef $convert;
 
}

###################################################
#
# Subroutine for plotting a FITS image in grayscale
#
###################################################

sub grayscale {

 # Arguments 

 my $array = float($_[0]); # image array
 my $crop = $_[1];         # pixels to crop in from each edge

 my $convert = $factor{$plot{'units'}};   # degrees to plot units

 my $base = $plot{'pixel_range'}->[0];      # base level of intensity
 my $peak = $plot{'pixel_range'}->[1];      # peak level of intensity

 if($base == 0 && $peak == 0) {
   $base = min($array);  $peak = max($array);
 }

 my($centx, $centy, $axisx, $axisy, $tr);

  # Get file info and tranformation information,
  # --  note that $convert is returned as a value that 
  #     converts from pixels to world units
 Get_Trans($array, $convert, $tr, $centx, $centy, $axisx, $axisy);

 $array = float($array);  # convert to float to be sure

 #
 # Plot it up...
 #

 if(!defined($crop)) {
   pggray($array->get_dataref,$axisx,$axisy,1,
        $axisx,1,$axisy,$peak,$base,$tr);
 } else {
   pggray($array->get_dataref,$axisx,$axisy,1+$crop->[0],
        $axisx-$crop->[1],1+$crop->[2],$axisy-$crop->[3], 
        $peak,$base,$tr);
 }

 if($plot{'wedge'}) {
  pgwedg($plot{'wedge_side'}."G", 
         $plot{'wedge_disp'},
         $plot{'wedge_width'},
         $peak,
         $base,
         $plot{'wedge_label'});
 }

 undef $array; 
 undef $axisx; undef $axisy; undef $centx; undef $centy;
 undef $base; undef $peak; undef $tr; undef $convert;
 
}

###################################################
#
# Subroutine for plotting a FITS image in color
#
###################################################

sub colorscale {

 # Arguments 

 my $array = $_[0]; # image array
 my $crop = $_[1];         # pixels to crop in from each edge

 my $convert = $factor{$plot{'units'}};   # degrees to plot units

 my $base = $plot{'pixel_range'}->[0];      # base level of intensity
 my $peak = $plot{'pixel_range'}->[1];      # peak level of intensity

 if($base == 0 && $peak == 0) {
   $base = min($array);  $peak = max($array);
 }

 my($centx, $centy, $axisx, $axisy, $tr);

  # Get file info and tranformation information,
  # --  note that $convert is returned as a value that 
  #     converts from pixels to world units
 Get_Trans($array, $convert, $tr, $centx, $centy, $axisx, $axisy);

 $array = float($array);  # convert to float to be sure

 #
 # Plot it up...
 #

 setcolor();   # to setup the currently selected color scheme

 if(!defined($crop)) {
   pgimag($array->get_dataref,$axisx,$axisy,1,
        $axisx,1,$axisy,$base,$peak,$tr);
 } else {
   pgimag($array->get_dataref,$axisx,$axisy,1+$crop->[0],
        $axisx-$crop->[1],1+$crop->[2],$axisy-$crop->[3], 
        $base,$peak,$tr);
 }

 if($plot{'wedge'}) {
  pgwedg($plot{'wedge_side'}."I", 
         $plot{'wedge_disp'},
         $plot{'wedge_width'},
         $base,
         $peak,
         $plot{'wedge_label'});
 }

 undef $array; 
 undef $axisx; undef $axisy; undef $centx; undef $centy;
 undef $base; undef $peak; undef $tr; undef $convert;
 
}

#####################################################
#
# Subroutine for setting the color scheme used
#
#####################################################

sub setcolor {

 my ($l, $r, $g, $b) = lut_data($plot{'color_type'},
                                $plot{'color_reverse'},
                                $plot{'color_ramp'});

 pgctab($l->get_dataref, $r->get_dataref, $g->get_dataref, 
        $b->get_dataref, $l->getdim(0), $plot{'color_contrast'}, 
                                        $plot{'color_bright'});

 undef $l; undef $r; undef $g; undef $b;

}


#####################################################
#
# Subroutine to start plotting
#
#####################################################

sub start_pgplot {

 pgbeg($_[0], $_[1], $_[2], $_[3]);

 # quick test to see if this is a postscript image, and if so, 
 # would the user would like the colormap reversed 
 my($dev, $a);
 pgqinf('DEV/TYPE',$dev,$a);
 if($dev =~ /\/*PS/) {                          
    print "Reverse colormap for postscript? (y/n): ";
     if(<STDIN> =~ /y/) { $plot{'color_reverse'} ^= 1; }
 }

 # setup some defaults
 pgscf($plot{'font'});          # use ROMAN lettering

}

#####################################################
#
# Subroutine to start plotting
#
#####################################################

sub begin_plot {

 my ($x1, $x2, $y1, $y2);

 $x1 = $plot{'plot_corners'}->[0];
 $x2 = $plot{'plot_corners'}->[1];
 $y1 = $plot{'plot_corners'}->[2];
 $y2 = $plot{'plot_corners'}->[3]; 

 # if not set, use image boundries
 if($x1 == 0 && $y1 == 0 && $x2 == 0 && $y2 == 0) {
   $x1 = $plot{'image_corners'}->[0];
   $x2 = $plot{'image_corners'}->[1];
   $y1 = $plot{'image_corners'}->[2];
   $y2 = $plot{'image_corners'}->[3]; 
 }

		    
 pgenv($x1,$x2,$y1,$y2,1,0);                  # define pgplot environment

} 


#####################################################
#
# Subroutine to finish plotting
#
#####################################################

sub finish_plot {

 pglabel("Relative R.A. ($plot{'units'})", 
         "Relative Declination ($plot{'units'})", $plot{'title'});    

 # redraw coordinate axis in case they were overwritten by 
 # the colorscale plot
 my ($x1, $x2, $y1, $y2);

 $x1 = $plot{'plot_corners'}->[0];
 $x2 = $plot{'plot_corners'}->[1];
 $y1 = $plot{'plot_corners'}->[2];
 $y2 = $plot{'plot_corners'}->[3]; 

 # if not set, use image boundries
 if($x1 == 0 && $y1 == 0 && $x2 == 0 && $y2 == 0) {
   $x1 = $plot{'image_corners'}->[0];
   $x2 = $plot{'image_corners'}->[1];
   $y1 = $plot{'image_corners'}->[2];
   $y2 = $plot{'image_corners'}->[3]; 
 }

 pgswin($x1,$x2,$y1,$y2);
 pgbox('BCNST', 0.0, 0, 'BCNST', 0.0, 0);

} 1;









