#!/usr/bin/perl

##########################################################
#
# example2.pl 
#
# This file contains a simple example of how the 
# subroutines in "FITSplot.pl" might be used to
# to create a complicated set of plots on one page.
# See other example files for simpler plots with
# more explanation.
#
# Defaults set in the "FITSplot.pl" file can be
# modified by specifying the appropriate $plot
# variable here.  See the README file for details.
#
# Finally, note that the subroutines in "FITSplot.pl" 
# are general, to control the details of pgplot (colors, 
# line styles, fonts, etc...) either change the defaults
# through the appropriate $plot variable or simply use
# the relevant pgplot command(s).
#
#  - Dan Homan, May 2002
#
##########################################################

require "FITSplot.pl";

##########################################################
#
# Setup some variables
#
##########################################################

my $Ifile = '3c120.ipol.gz';  # total intensity FITS file
my $Pfile = '3c120.ppol.gz';  # polarized intensity FITS file
my $Xfile = '3c120.pang.gz';  # polarization angle FITS file

$plot{'tick_rotate'} = -3;   # polarization angle correction (degrees)
$plot{'tick_length'} = 0.2;  # scale length of tick marks in plot units

my $pcut = 0.002;             # cutoff levels for tick mark plotting
my $icut = 0.002;             #  and fractional polarization image

my $Xint = 0;            # set the interval between the three
my $Yint = 3.5;          # images on the output page -- these
                         # will be use to set the $plot{'offset'} variable
                         # before each plotting command below  

my @conts = loglevs(0.002,sqrt(2));   # define an array of contour
                                      #   levels starting at 0.002 Jy
                                      #   in steps of sqrt(2)
                                      #   -- this subroutine is just
                                      #      for convience, contours
                                      #      can be simply defined like
                                      #      @conts = (1, 2, 3, 4);  

my $nlevs = @conts;                   # get length of the array for
                                      #   the number of contours 

my @conts2 = loglevs(0.002,2);        # same as above, but for x2
my $nlevs2 = @conts2;                 #   contours

#########################################################
#
# Open FITS files and slice the image arrays down to size
#
#########################################################

my ($Iarray, $Parray, $Xarray, $Farray);  # define image arrays in advance

# define sub-region of interest in the image, default is full image,
#   format is [x1, x2, y1, y2]
$plot{'image_corners'} = [ 1.5, -9.0, -4.8, 2.2 ];  

$Iarray = Load_FITS($Ifile);     # Read in the FITS files, this returns
$Parray = Load_FITS($Pfile);     #  arrays cut to 'image_corners'     
$Xarray = Load_FITS($Xfile);

$Farray = MakeF($Iarray, $Parray, $icut, $pcut); # Make fractional pol.
 
##########################################################
#
# Begin the plot....
#
##########################################################

start_pgplot(0,'?',1,1);   # startup PGPLOT, arguments are the same
                           #  as for the pgbeg command in PGPLOT

# Define corners for the plotting frame.  Note that these
#   are different from the 'image_corners'. 
$plot{'plot_corners'} = [ 2, -9.0, -11, 3 ]; 

begin_plot();              # subroutine to begin current plot

#  First is a color F-pol image that will be 
#  overlayed by fractor of 2 I contours, but we make 
#  the color image before other graphics because otherwise 
#  it will overwrite the other graphics. 

$plot{'offset'} = [ -2*$Xint, -2*$Yint ];  # set offset for plotting
$plot{'wedge_side'} = 'R';                 # set side where color wedge goes   
colorscale($Farray);                       # plot color F-pol image
contour($Iarray,\@conts2,$nlevs2);         # overplot with x2 I contours

#  Next plot the $Iarray contours at the top of the page

$plot{'offset'} = [ 0, 0 ];                # reset offset to zero
contour($Iarray,\@conts,$nlevs);           # Plot P contours 

plotbeam($Iarray);         # plot the beam figure for $Iarray image

#  Next plot the $Parray contours in the middle, with a single
#  contour of $Iarray around it 

$plot{'offset'} = [ -$Xint, -$Yint ];  # set offset again
contour($Parray,\@conts,$nlevs);       # Plot P contours 
contour($Iarray,[ 0.002 ], 1);         # single I contour around P

ticks($Parray, $Xarray, $pcut);  # plot polarization ticks of uniform
                                 #   size.  Variable sized ticks can
                                 #   be plotted by specifying a 4th
                                 #   argument to the function which
                                 #   would be an array of lengths.

# write component locations (and labels) from another file
#  and place them intelligently...

open(DATA, "3c120.dat") || die "cannot open data file";
my ($cx, $cy, $cname, $cepoch, $ci, $cq, $cu);
my ($cx1, $cy1);
my $clast = +50; my $symbol = -4;
while(<DATA>) {
 ($cepoch, $cname, $ci, $cq, $cu, $cx, $cy) = split (' ');
 if($cepoch == 1996.05 && $cx < $plot{'image_corners'}->[0] 
                       && $cx > $plot{'image_corners'}->[1]) {
   pgpt(1,$cx, $cy, $symbol);
   $cy1 = $plot{'image_corners'}->[3]-0.1;
   $cx1 = $cx + 0.2;
   if($clast < $cx + 1.5) {
    $cx1 = $clast - 1.5;
   }
   pgtext($cx1, $cy1, $cname); 
   pgsls(2);
   pgmove($cx, $cy);
   pgdraw($cx1-0.2, $cy1-0.1);
   pgsls(1);
   $clast = $cx1;
 }
}
close(DATA);


$plot{'title'} = "3C120";
finish_plot();             # subroutine to finish the plot

pgend();                   # end pgplot 

exit;


